import React from 'react';

function createContext(obj) {
  var Context = toContext(obj);

  function toContext(obj) {
    if (isContext(obj)) return obj;
    if (Array.isArray(obj)) return arrayToContext(obj);
    if (isObject(obj)) return objectToContext(obj);
    return React.createContext(obj);
  }

  function isContext(value) {
    return !!value.Provider && !!value.Consumer;
  }

  function arrayToContext(obj) {
    var contexts = obj.map(toContext);

    var Provider = function Provider(_ref) {
      var value = _ref.value,
          children = _ref.children;

      return contexts.reduce(function (child, context, i) {
        return React.createElement(
          context.Provider,
          { value: value[i] },
          child
        );
      }, children);
    };

    var Consumer = function Consumer(_ref2) {
      var children = _ref2.children;

      var values = [];

      var curry = function curry(i) {
        var context = contexts[i];

        return React.createElement(
          context.Consumer,
          null,
          function (value) {
            values.push(value);

            if (values.length === contexts.length) {
              return children(values);
            } else {
              return curry(i + 1);
            }
          }
        );
      };

      return curry(0);
    };

    return { Provider: Provider, Consumer: Consumer };
  }

  function objectToContext(obj) {
    var contexts = {};
    var keys = Object.keys(obj);

    keys.forEach(function (key) {
      contexts[key] = toContext(obj[key]);
    });

    var Provider = function Provider(_ref3) {
      var value = _ref3.value,
          children = _ref3.children;

      return keys.reduce(function (child, key) {
        var context = contexts[key];
        return React.createElement(
          context.Provider,
          { value: value[key] },
          child
        );
      }, children);
    };

    var Consumer = function Consumer(_ref4) {
      var children = _ref4.children;

      var values = {};

      var curry = function curry(i) {
        var key = keys[i];
        var context = contexts[key];

        return React.createElement(
          context.Consumer,
          null,
          function (value) {
            values[key] = value;

            if (keys.length === i + 1) {
              return children(values);
            } else {
              return curry(i + 1);
            }
          }
        );
      };

      return curry(0);
    };

    return { Provider: Provider, Consumer: Consumer };
  }

  function isObject(obj) {
    return Object === obj.constructor;
  }

  return Context;
}

export default createContext;
